package com.shiyanlou.core.token.server;

import com.shiyanlou.core.properties.OAuth2ClientProperties;
import com.shiyanlou.core.properties.SecurityProperties;
import org.apache.commons.lang.ArrayUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.security.oauth2.config.annotation.builders.InMemoryClientDetailsServiceBuilder;
import org.springframework.security.oauth2.config.annotation.configurers.ClientDetailsServiceConfigurer;
import org.springframework.security.oauth2.config.annotation.web.configuration.AuthorizationServerConfigurerAdapter;
import org.springframework.security.oauth2.config.annotation.web.configuration.EnableAuthorizationServer;
import org.springframework.security.oauth2.config.annotation.web.configurers.AuthorizationServerEndpointsConfigurer;
import org.springframework.security.oauth2.provider.token.TokenStore;
import org.springframework.security.oauth2.provider.token.store.JwtAccessTokenConverter;

/**
 * @ClassName ShiyanlouAuthorizationServerConfig
 * @Description 认证服务器配置
 * @Date 2020/11/21 19:15
 * @Modify ...
 */
@Configuration
@EnableAuthorizationServer
public class ShiyanlouAuthorizationServerConfig extends AuthorizationServerConfigurerAdapter {

    @Autowired
    private SecurityProperties securityProperties;

    @Autowired
    private PasswordEncoder passwordEncoder;

    @Autowired
    private UserDetailsService userDetailsService;

    @Autowired
    private AuthenticationManager authenticationManager;

    @Autowired
    private TokenStore tokenStore;

    // 只有当使用jwt的时候才会有该对象
    @Autowired(required = false)
    private JwtAccessTokenConverter jwtAccessTokenConverter;


    @Override
    public void configure(AuthorizationServerEndpointsConfigurer endpoints) throws Exception {
        endpoints
                .tokenStore(tokenStore)
                .authenticationManager(authenticationManager)
                .userDetailsService(userDetailsService);

        if (jwtAccessTokenConverter != null) {
            endpoints.accessTokenConverter(jwtAccessTokenConverter);
        }
    }


    /*
     * 功能描述：客户端配置
     * 入参：[clients]
     * 返回值：void
     */
    @Override
    public void configure(ClientDetailsServiceConfigurer clients) throws Exception {



        InMemoryClientDetailsServiceBuilder builder = clients.inMemory();
        if (ArrayUtils.isNotEmpty(securityProperties.getOauth2().getClients())) {
            for (OAuth2ClientProperties config : securityProperties.getOauth2().getClients()) {
                builder.withClient(config.getClientId())// 设置clientid
                // Spring security5中新增加了加密方式，并把原有的spring security的密码存储格式改了
                        .secret(passwordEncoder.encode(config.getClientSecret()))// 设置clientsecret
                        .accessTokenValiditySeconds(config.getAccessTokenValidateSeconds())// 设置令牌过期时间
                        .authorizedGrantTypes("refresh_token", "authorization_code", "password")// 允许的授权模式
                        .refreshTokenValiditySeconds(2592000)// 设置刷新令牌的过期时间
                        .scopes("all", "write", "read");// 配置oauth能获取的权限
            }
        }
    }
}
